//******************************************************************************
//Name          : Statistic.cs
//Description   : Implementation to experience the beauty and the pitfalls of Statistics
//Author        : Dr. Oliver Kluge, 90411 Nuernberg, Germany
//Copyright(C)  : Oliver Kluge 2014 
//------------------------------------------------------------------------------
//******************************************************************************

using System;


class Statistic
{
    const int samples = 512;                        // Stichprobenumfang

    public double[] x = new double[samples];        // Stichprobe
    public double[] sort = new double[samples];	    // sortierte Stichprobe
    private MyRandom.Random noise = new MyRandom.Random(0.71036, 0.23906);

    public double mw1;      // Mittelwert
    public double mw2;      // Mittelwert (rekursiv)
    public double med;      // Median
    public double spw;      // Spannweite
    public double mab;      // Mittl. abs. Abweichung vom Mittelw.
    public double str1;     // Streuung
    public double str2;     // Streuung (2. Variante)
    public double var1;     // Varianz
    public double var2;     // Varianz (2. Variante)
    public double schi;     // Schiefe
    public double kurt;     // Kurtosis

    public void CreateSignal(int select)
    {
        if (select == 1)
        {
            // Gleichverteilung
            for (int k = 0; k < samples; k++)
            {
                x[k] = noise.Uniform1();
            }
        }

        if (select == 2)
        {
            // Dreiecksverteilung
            for (int k = 0; k < samples; k++)
            {
                x[k] = noise.Uniform1() + noise.Uniform2();
            }
        }

        if (select == 3)
        {
            // Normalverteilung
            for (int k = 0; k < samples; k++)
            {
                x[k] = noise.Normal(0, 1);
            }
        }

        if (select == 4)
        {
            // Exponentialverteilung
            for (int k = 0; k < samples; k++)
            {
                x[k] = noise.Exponential(1);
            }
        }
    }

    public void QuickSort(int start, int stop, double[] sort)
    {
        int von     = 0;
        int bis     = 0;
        double wert = 0;        // Pivotelement
        double temp = 0;

        von = start;
        bis = stop;
        // Vergleichswert aus Intervallmitte
        wert = sort[(int)((von+bis)/2.0)];

        do 
        {
            while (sort[von] < wert) von++;
            while (sort[bis] > wert) bis--;

            if (von <= bis)
            {
                temp = sort[von];
                sort[von] = sort[bis];
                sort[bis] = temp;
                von++;
                bis--;
            }
        } while (von <= bis);

        if (start < bis) QuickSort(start, bis, sort);
        if (stop > von)  QuickSort(von, stop, sort);
    }

    public double Median(int samples)
    {
        double median = 0;

        // Arbeitskopie der Stichprobe anlegen
        for (int i=0; i<samples; i++)
        {
            sort[i] = x[i];
        }

        // Alle k Werte x(0)..x(k-1) aufsteigend sortieren
        QuickSort(0, samples-1, sort);

        // Ist samples gerade oder ungerade?
        if (samples % 2 == 0)
        {
            median = (sort[samples/2 - 1] + sort[samples/2]) / 2.0;
        }
        else
        {
            median = sort[(samples-1)/2];
        }
        return median;
    }

    public void ComputeStatistic()
    {
        // Mittelwert
        mw1 = 0;
        for (int i=0; i<samples; i++)
        {
            mw1 += x[i];
        }
        mw1 /= samples;

        // Mittelwert rekursiv
        mw2 = 0;
        for (int i=0; i<samples; i++)
        {
            mw2 = (i*mw2 + x[i])/(i+1);  // i=0..N-1 instead of i=1..N
        }

        // Median
        med = Median(samples);

        // Spannweite
        spw = sort[samples-1] - sort[0];

        // Mittlere absolute Abweichung vom Mittelwert
        mab = 0;
        for (int i=0; i<samples; i++)
        {
            mab += Math.Abs(x[i]-mw1);
        }
        mab /= samples;

        // Varianz
        if (samples > 1)                // wegen Varianz=0 bei N=1
        {
            var1 = 0;
            for (int i=0; i<samples; i++)
            {
                var1 += (x[i]-mw1) * (x[i]-mw1);
            }
            var1 /= (samples - 1);
        }

        // Varianz (2. Variante)
        if (samples > 1)                // wegen Varianz=0 bei N=1
        {
            var2 = 0;
            for (int i=0; i<samples; i++)
            {
                var2 += (x[i] * x[i]);
            }
            var2 -= samples * (mw1 * mw1);
            var2 /= (samples - 1);
        }

        // Streuung
        str1  = Math.Sqrt(var1);

        // Streuung (2. Variante)
        str2  = Math.Sqrt(var2);

        // Schiefe
        if (samples > 1)                // wegen Streuung=0 bei N=1
        {
            schi = 0;
            for (int i=0; i<samples; i++)
            {
                schi += (x[i]-mw1) * (x[i]-mw1) * (x[i]-mw1);
            }
            schi /= (samples * str1 * str1 * str1);
        }

        // Kurtosis
        if (samples > 1)                // wegen Streuung=0 bei N=1
        {
            kurt = 0;
            for (int i=0; i<samples; i++)
            {
                kurt += (x[i]-mw1) * (x[i]-mw1) * (x[i]-mw1) * (x[i]-mw1);
            }
            kurt /= (samples * str1 * str1 * str1 * str1);
        }
    }
}


class MainApp
{
    [STAThread]
    static void Main()
    {
        string userinput = "";
        int select = -1;

        while (select <= 4)
        {
            Statistic myStatistic = new Statistic();

            Console.Write("Bitte Stichprobe auswhlen:\n\n");
            Console.Write("Gleichverteilung      ->  1\n");
            Console.Write("Dreiecksverteilung    ->  2\n");
            Console.Write("Normalverteilung      ->  3\n");
            Console.Write("Exponentialverteilung ->  4\n");
            Console.WriteLine();

            userinput = Console.ReadLine();
            select = Convert.ToInt32(userinput);

            myStatistic.CreateSignal(select);
            myStatistic.ComputeStatistic();

            Console.WriteLine("Mittewert        = {0}", myStatistic.mw1);
            Console.WriteLine("rek. Mittewert   = {0}", myStatistic.mw2);
            Console.WriteLine("Median           = {0}", myStatistic.med);
            Console.WriteLine("Spannweite       = {0}", myStatistic.spw);
            Console.WriteLine("mittl. abs. Abw. = {0}", myStatistic.mab);
            Console.WriteLine("Streuung         = {0}", myStatistic.str1);
            Console.WriteLine("Streuung2        = {0}", myStatistic.str2);
            Console.WriteLine("Varianz          = {0}", myStatistic.var1);
            Console.WriteLine("Varianz2         = {0}", myStatistic.var2);
            Console.WriteLine("Schiefe          = {0}", myStatistic.schi);
            Console.WriteLine("Kurtosis         = {0}", myStatistic.kurt);

            Console.ReadLine();
            for (int i = 0; i < 20; i++) Console.WriteLine();
        }

        Console.WriteLine();
        Console.WriteLine("Ende...<ENTER>");
        Console.ReadLine();
    }
}
